@@ -13,6 +13,7 @@
 # limitations under the License.
 
 """Tools for representing JavaScript code in BSON."""
+from __future__ import annotations
 
 from collections.abc import Mapping as _Mapping
 from typing import Any, Mapping, Optional, Type, Union
@@ -30,15 +31,14 @@ class Code(str):
     keyword argument it will override any setting for that variable in
     the `scope` dictionary.
 
-    :Parameters:
-      - `code`: A string containing JavaScript code to be evaluated or another
+    :param code: A string containing JavaScript code to be evaluated or another
         instance of Code. In the latter case, the scope of `code` becomes this
         Code's :attr:`scope`.
-      - `scope` (optional): dictionary representing the scope in which
+    :param scope: dictionary representing the scope in which
         `code` should be evaluated - a mapping from identifiers (as
         strings) to values. Defaults to ``None``. This is applied after any
         scope associated with a given `code` above.
-      - `**kwargs` (optional): scope variables can also be passed as
+    :param kwargs: scope variables can also be passed as
         keyword arguments. These are applied after `scope` and `code`.
 
     .. versionchanged:: 3.4
@@ -50,13 +50,13 @@ class Code(str):
     __scope: Union[Mapping[str, Any], None]
 
     def __new__(
-        cls: Type["Code"],
-        code: Union[str, "Code"],
+        cls: Type[Code],
+        code: Union[str, Code],
         scope: Optional[Mapping[str, Any]] = None,
         **kwargs: Any,
-    ) -> "Code":
+    ) -> Code:
         if not isinstance(code, str):
-            raise TypeError("code must be an instance of str")
+            raise TypeError(f"code must be an instance of str, not {type(code)}")
 
         self = str.__new__(cls, code)
 
@@ -67,7 +67,7 @@ class Code(str):
 
         if scope is not None:
             if not isinstance(scope, _Mapping):
-                raise TypeError("scope must be an instance of dict")
+                raise TypeError(f"scope must be an instance of dict, not {type(scope)}")
             if self.__scope is not None:
                 self.__scope.update(scope)  # type: ignore
             else:
