@@ -13,6 +13,7 @@
 # limitations under the License.
 
 """Tools for specifying BSON codec options."""
+from __future__ import annotations
 
 import abc
 import datetime
@@ -22,7 +23,6 @@ from typing import (
     TYPE_CHECKING,
     Any,
     Callable,
-    Dict,
     Generic,
     Iterable,
     Mapping,
@@ -57,7 +57,7 @@ class TypeEncoder(abc.ABC):
     Codec classes must implement the ``python_type`` attribute, and the
     ``transform_python`` method to support encoding.
 
-    See :ref:`custom-type-type-codec` documentation for an example.
+    See `encode data with type codecs <https://www.mongodb.com/docs/languages/python/pymongo-driver/current/data-formats/custom-types/type-codecs/#encode-data-with-type-codecs>`_ documentation for an example.
     """
 
     @abc.abstractproperty
@@ -76,7 +76,7 @@ class TypeDecoder(abc.ABC):
     Codec classes must implement the ``bson_type`` attribute, and the
     ``transform_bson`` method to support decoding.
 
-    See :ref:`custom-type-type-codec` documentation for an example.
+    See `encode data with type codecs <https://www.mongodb.com/docs/languages/python/pymongo-driver/current/data-formats/custom-types/type-codecs/#encode-data-with-type-codecs>`_ documentation for an example.
     """
 
     @abc.abstractproperty
@@ -98,7 +98,7 @@ class TypeCodec(TypeEncoder, TypeDecoder):
     ``bson_type`` attribute, and the ``transform_bson`` method to support
     decoding.
 
-    See :ref:`custom-type-type-codec` documentation for an example.
+    See `encode data with type codecs <https://www.mongodb.com/docs/languages/python/pymongo-driver/current/data-formats/custom-types/type-codecs/#encode-data-with-type-codecs>`_ documentation for an example.
     """
 
 
@@ -118,18 +118,17 @@ class TypeRegistry:
       >>> type_registry = TypeRegistry([Codec1, Codec2, Codec3, ...],
       ...                              fallback_encoder)
 
-    See :ref:`custom-type-type-registry` documentation for an example.
+    See `add codec to the type registry <https://www.mongodb.com/docs/languages/python/pymongo-driver/current/data-formats/custom-types/type-codecs/#add-codec-to-the-type-registry>`_ documentation for an example.
 
-    :Parameters:
-      - `type_codecs` (optional): iterable of type codec instances. If
+    :param type_codecs: iterable of type codec instances. If
         ``type_codecs`` contains multiple codecs that transform a single
         python or BSON type, the transformation specified by the type codec
         occurring last prevails. A TypeError will be raised if one or more
         type codecs modify the encoding behavior of a built-in :mod:`bson`
         type.
-      - `fallback_encoder` (optional): callable that accepts a single,
+    :param fallback_encoder: callable that accepts a single,
         unencodable python value and transforms it into a type that
-        :mod:`bson` can encode. See :ref:`fallback-encoder-callable`
+        :mod:`bson` can encode. See `define a fallback encoder <https://www.mongodb.com/docs/languages/python/pymongo-driver/current/data-formats/custom-types/type-codecs/#define-a-fallback-encoder>`_
         documentation for an example.
     """
 
@@ -140,8 +139,8 @@ class TypeRegistry:
     ) -> None:
         self.__type_codecs = list(type_codecs or [])
         self._fallback_encoder = fallback_encoder
-        self._encoder_map: Dict[Any, Any] = {}
-        self._decoder_map: Dict[Any, Any] = {}
+        self._encoder_map: dict[Any, Any] = {}
+        self._decoder_map: dict[Any, Any] = {}
 
         if self._fallback_encoder is not None:
             if not callable(fallback_encoder):
@@ -161,6 +160,16 @@ class TypeRegistry:
                     f"Expected an instance of {TypeEncoder.__name__}, {TypeDecoder.__name__}, or {TypeCodec.__name__}, got {codec!r} instead"
                 )
 
+    @property
+    def codecs(self) -> list[TypeEncoder | TypeDecoder | TypeCodec]:
+        """The list of type codecs in this registry."""
+        return self.__type_codecs
+
+    @property
+    def fallback_encoder(self) -> Optional[_Fallback]:
+        """The fallback encoder in this registry."""
+        return self._fallback_encoder
+
     def _validate_type_encoder(self, codec: _Codec) -> None:
         from bson import _BUILT_IN_TYPES
 
@@ -168,7 +177,7 @@ class TypeRegistry:
             if issubclass(cast(TypeCodec, codec).python_type, pytype):
                 err_msg = (
                     "TypeEncoders cannot change how built-in types are "
-                    "encoded (encoder {} transforms type {})".format(codec, pytype)
+                    f"encoded (encoder {codec} transforms type {pytype})"
                 )
                 raise TypeError(err_msg)
 
@@ -236,7 +245,7 @@ class _BaseCodecOptions(NamedTuple):
 
 if TYPE_CHECKING:
 
-    class CodecOptions(Tuple, Generic[_DocumentType]):
+    class CodecOptions(Tuple[_DocumentType], Generic[_DocumentType]):
         document_class: Type[_DocumentType]
         tz_aware: bool
         uuid_representation: int
@@ -246,7 +255,7 @@ if TYPE_CHECKING:
         datetime_conversion: Optional[int]
 
         def __new__(
-            cls: Type["CodecOptions"],
+            cls: Type[CodecOptions[_DocumentType]],
             document_class: Optional[Type[_DocumentType]] = ...,
             tz_aware: bool = ...,
             uuid_representation: Optional[int] = ...,
@@ -254,28 +263,28 @@ if TYPE_CHECKING:
             tzinfo: Optional[datetime.tzinfo] = ...,
             type_registry: Optional[TypeRegistry] = ...,
             datetime_conversion: Optional[int] = ...,
-        ) -> "CodecOptions[_DocumentType]":
+        ) -> CodecOptions[_DocumentType]:
             ...
 
         # CodecOptions API
-        def with_options(self, **kwargs: Any) -> "CodecOptions[_DocumentType]":
+        def with_options(self, **kwargs: Any) -> CodecOptions[Any]:
             ...
 
         def _arguments_repr(self) -> str:
             ...
 
-        def _options_dict(self) -> Dict[Any, Any]:
+        def _options_dict(self) -> dict[Any, Any]:
             ...
 
         # NamedTuple API
         @classmethod
-        def _make(cls, obj: Iterable) -> "CodecOptions[_DocumentType]":
+        def _make(cls, obj: Iterable[Any]) -> CodecOptions[_DocumentType]:
             ...
 
-        def _asdict(self) -> Dict[str, Any]:
+        def _asdict(self) -> dict[str, Any]:
             ...
 
-        def _replace(self, **kwargs: Any) -> "CodecOptions[_DocumentType]":
+        def _replace(self, **kwargs: Any) -> CodecOptions[_DocumentType]:
             ...
 
         _source: str
@@ -318,36 +327,35 @@ else:
               >>> doc._id
               ObjectId('5b3016359110ea14e8c58b93')
 
-            See :doc:`/examples/datetimes` for examples using the `tz_aware` and
+            See `Dates and Times <https://www.mongodb.com/docs/languages/python/pymongo-driver/current/data-formats/dates-and-times/#dates-and-times>`_ for examples using the `tz_aware` and
             `tzinfo` options.
 
-            See :doc:`/examples/uuid` for examples using the `uuid_representation`
+            See `UUID <https://www.mongodb.com/docs/languages/python/pymongo-driver/current/data-formats/uuid/#universally-unique-ids--uuids->`_ for examples using the `uuid_representation`
             option.
 
-            :Parameters:
-              - `document_class`: BSON documents returned in queries will be decoded
+            :param document_class: BSON documents returned in queries will be decoded
                 to an instance of this class. Must be a subclass of
                 :class:`~collections.abc.MutableMapping`. Defaults to :class:`dict`.
-              - `tz_aware`: If ``True``, BSON datetimes will be decoded to timezone
+            :param tz_aware: If ``True``, BSON datetimes will be decoded to timezone
                 aware instances of :class:`~datetime.datetime`. Otherwise they will be
                 naive. Defaults to ``False``.
-              - `uuid_representation`: The BSON representation to use when encoding
+            :param uuid_representation: The BSON representation to use when encoding
                 and decoding instances of :class:`~uuid.UUID`. Defaults to
                 :data:`~bson.binary.UuidRepresentation.UNSPECIFIED`. New
                 applications should consider setting this to
                 :data:`~bson.binary.UuidRepresentation.STANDARD` for cross language
-                compatibility. See :ref:`handling-uuid-data-example` for details.
-              - `unicode_decode_error_handler`: The error handler to apply when
+                compatibility. See `UUID representations <https://www.mongodb.com/docs/languages/python/pymongo-driver/current/data-formats/uuid/#universally-unique-ids--uuids->`_ for details.
+            :param unicode_decode_error_handler: The error handler to apply when
                 a Unicode-related error occurs during BSON decoding that would
                 otherwise raise :exc:`UnicodeDecodeError`. Valid options include
                 'strict', 'replace', 'backslashreplace', 'surrogateescape', and
                 'ignore'. Defaults to 'strict'.
-              - `tzinfo`: A :class:`~datetime.tzinfo` subclass that specifies the
+            :param tzinfo: A :class:`~datetime.tzinfo` subclass that specifies the
                 timezone to/from which :class:`~datetime.datetime` objects should be
                 encoded/decoded.
-              - `type_registry`: Instance of :class:`TypeRegistry` used to customize
+            :param type_registry: Instance of :class:`TypeRegistry` used to customize
                 encoding and decoding behavior.
-              - `datetime_conversion`: Specifies how UTC datetimes should be decoded
+            :param datetime_conversion: Specifies how UTC datetimes should be decoded
                 within BSON. Valid options include 'datetime_ms' to return as a
                 DatetimeMS, 'datetime' to return as a datetime.datetime and
                 raising a ValueError for out-of-range values, 'datetime_auto' to
@@ -372,7 +380,7 @@ else:
             super().__init__()
 
         def __new__(
-            cls: Type["CodecOptions"],
+            cls: Type[CodecOptions],
             document_class: Optional[Type[Mapping[str, Any]]] = None,
             tz_aware: bool = False,
             uuid_representation: Optional[int] = UuidRepresentation.UNSPECIFIED,
@@ -380,7 +388,7 @@ else:
             tzinfo: Optional[datetime.tzinfo] = None,
             type_registry: Optional[TypeRegistry] = None,
             datetime_conversion: Optional[DatetimeConversion] = DatetimeConversion.DATETIME,
-        ) -> "CodecOptions":
+        ) -> CodecOptions:
             doc_class = document_class or dict
             # issubclass can raise TypeError for generic aliases like SON[str, Any].
             # In that case we can use the base class for the comparison.
@@ -403,17 +411,23 @@ else:
                     "uuid_representation must be a value from bson.binary.UuidRepresentation"
                 )
             if not isinstance(unicode_decode_error_handler, str):
-                raise ValueError("unicode_decode_error_handler must be a string")
+                raise ValueError(
+                    f"unicode_decode_error_handler must be a string, not {type(unicode_decode_error_handler)}"
+                )
             if tzinfo is not None:
                 if not isinstance(tzinfo, datetime.tzinfo):
-                    raise TypeError("tzinfo must be an instance of datetime.tzinfo")
+                    raise TypeError(
+                        f"tzinfo must be an instance of datetime.tzinfo, not {type(tzinfo)}"
+                    )
                 if not tz_aware:
                     raise ValueError("cannot specify tzinfo without also setting tz_aware=True")
 
             type_registry = type_registry or TypeRegistry()
 
             if not isinstance(type_registry, TypeRegistry):
-                raise TypeError("type_registry must be an instance of TypeRegistry")
+                raise TypeError(
+                    f"type_registry must be an instance of TypeRegistry, not {type(type_registry)}"
+                )
 
             return tuple.__new__(
                 cls,
@@ -452,7 +466,7 @@ else:
                 )
             )
 
-        def _options_dict(self) -> Dict[str, Any]:
+        def _options_dict(self) -> dict[str, Any]:
             """Dictionary of the arguments used to create this object."""
             # TODO: PYTHON-2442 use _asdict() instead
             return {
@@ -468,7 +482,7 @@ else:
         def __repr__(self) -> str:
             return f"{self.__class__.__name__}({self._arguments_repr()})"
 
-        def with_options(self, **kwargs: Any) -> "CodecOptions":
+        def with_options(self, **kwargs: Any) -> CodecOptions:
             """Make a copy of this CodecOptions, overriding some options::
 
                 >>> from bson.codec_options import DEFAULT_CODEC_OPTIONS
@@ -485,10 +499,10 @@ else:
             return CodecOptions(**opts)
 
 
-DEFAULT_CODEC_OPTIONS: "CodecOptions[Dict[str, Any]]" = CodecOptions()
+DEFAULT_CODEC_OPTIONS: CodecOptions[dict[str, Any]] = CodecOptions()
 
 
-def _parse_codec_options(options: Any) -> CodecOptions:
+def _parse_codec_options(options: Any) -> CodecOptions[Any]:
     """Parse BSON codec options."""
     kwargs = {}
     for k in set(options) & {
