@@ -4,7 +4,7 @@
 # may not use this file except in compliance with the License.  You
 # may obtain a copy of the License at
 #
-# http://www.apache.org/licenses/LICENSE-2.0
+# https://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
@@ -12,7 +12,10 @@
 # implied.  See the License for the specific language governing
 # permissions and limitations under the License.
 
-"""Represent a deployment of MongoDB servers."""
+"""Represent a deployment of MongoDB servers.
+
+.. seealso:: This module is compatible with both the synchronous and asynchronous PyMongo APIs.
+"""
 from __future__ import annotations
 
 from random import sample
@@ -30,8 +33,8 @@ from typing import (
 from bson.min_key import MinKey
 from bson.objectid import ObjectId
 from pymongo import common
-from pymongo.errors import ConfigurationError
-from pymongo.read_preferences import ReadPreference, _AggWritePref, _ServerMode
+from pymongo.errors import ConfigurationError, PyMongoError
+from pymongo.read_preferences import Primary, ReadPreference, _AggWritePref, _ServerMode
 from pymongo.server_description import ServerDescription
 from pymongo.server_selectors import Selection
 from pymongo.server_type import SERVER_TYPE
@@ -321,6 +324,17 @@ class TopologyDescription:
             description = self.server_descriptions().get(address)
             return [description] if description else []
 
+        # Primary selection fast path.
+        if self.topology_type == TOPOLOGY_TYPE.ReplicaSetWithPrimary and type(selector) is Primary:
+            for sd in self._server_descriptions.values():
+                if sd.server_type == SERVER_TYPE.RSPrimary:
+                    sds = [sd]
+                    if custom_selector:
+                        sds = custom_selector(sds)
+                    return sds
+            # No primary found, return an empty list.
+            return []
+
         selection = Selection.from_topology_description(self)
         # Ignore read preference for sharded clusters.
         if self.topology_type != TOPOLOGY_TYPE.Sharded:
@@ -555,12 +569,16 @@ def _update_rs_from_primary(
         return _check_has_primary(sds), replica_set_name, max_set_version, max_election_id
 
     if server_description.max_wire_version is None or server_description.max_wire_version < 17:
-        new_election_tuple: tuple = (server_description.set_version, server_description.election_id)
-        max_election_tuple: tuple = (max_set_version, max_election_id)
+        new_election_tuple: tuple = (server_description.set_version, server_description.election_id)  # type: ignore[type-arg]
+        max_election_tuple: tuple = (max_set_version, max_election_id)  # type: ignore[type-arg]
         if None not in new_election_tuple:
             if None not in max_election_tuple and new_election_tuple < max_election_tuple:
                 # Stale primary, set to type Unknown.
-                sds[server_description.address] = server_description.to_unknown()
+                sds[server_description.address] = server_description.to_unknown(
+                    PyMongoError(
+                        f"primary marked stale due to electionId/setVersion mismatch, {new_election_tuple} is stale compared to {max_election_tuple}"
+                    )
+                )
                 return _check_has_primary(sds), replica_set_name, max_set_version, max_election_id
             max_election_id = server_description.election_id
 
@@ -575,7 +593,11 @@ def _update_rs_from_primary(
         max_election_safe = tuple(MinKey() if i is None else i for i in max_election_tuple)
         if new_election_safe < max_election_safe:
             # Stale primary, set to type Unknown.
-            sds[server_description.address] = server_description.to_unknown()
+            sds[server_description.address] = server_description.to_unknown(
+                PyMongoError(
+                    f"primary marked stale due to electionId/setVersion mismatch, {new_election_tuple} is stale compared to {max_election_tuple}"
+                )
+            )
             return _check_has_primary(sds), replica_set_name, max_set_version, max_election_id
         else:
             max_election_id = server_description.election_id
@@ -588,7 +610,9 @@ def _update_rs_from_primary(
             and server.address != server_description.address
         ):
             # Reset old primary's type to Unknown.
-            sds[server.address] = server.to_unknown()
+            sds[server.address] = server.to_unknown(
+                PyMongoError("primary marked stale due to discovery of newer primary")
+            )
 
             # There can be only one prior primary.
             break
